package skyjed;

import skyjed.gui.MainWindow;
import skyjed.save.SaveFile;
import skyjed.util.ExceptionHandler;

import javax.swing.*;
import java.io.*;
import java.util.Arrays;

public class SkyJEd {

    /**
     * Launch the application.
     */
    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
                    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                    new MainWindow();
                } catch (Exception e) {
                    ExceptionHandler.handleException(e, null);
                }
            }
        });
    }

    public static SaveFile openFile(File file) throws IOException {
        long length = file.length();
        checkFileLength(length, true);
        byte[] rawbuf = new byte[(int) length];
        try (DataInputStream fin = new DataInputStream(new FileInputStream(file))) {
            fin.readFully(rawbuf);
        }
        return new SaveFile(rawbuf);
    }

    public static void saveFile(File file, SaveFile save) throws IOException {
        try (FileOutputStream fout = new FileOutputStream(file)) {
            fout.write(save.toByteA());
        }
    }

    public static void truncateFile(File file) throws IOException {
        long length = file.length();
        checkFileLength(length, false);
        byte[] rawbuf = new byte[SaveFile.MIN_FILE_LENGTH];
        try (DataInputStream fin = new DataInputStream(new FileInputStream(file))) {
            fin.readFully(rawbuf);
        }
        try (FileOutputStream fout = new FileOutputStream(file)) {
            fout.write(rawbuf);
        }
    }

    private static void checkFileLength(long length, boolean upper) throws IOException {
        if(length < SaveFile.MIN_FILE_LENGTH) {
            throw new IOException("File is too short for a pkmn md sky save");
        }
        if(upper && length >= 4*1024*1024) { // 4MB
            throw new IOException(
                    "File is way too big for a save file.\n" +
                    "If you're sure you may want to truncate this file first.");
        }
    }

    public static boolean testLoadSave(File file) throws IOException {
        long length = file.length();
        checkFileLength(length, true);
        byte[] rawbuf = new byte[SaveFile.MIN_FILE_LENGTH];
        try (DataInputStream fin = new DataInputStream(new FileInputStream(file))) {
            fin.readFully(rawbuf);
        }

        byte[] tbuf = Arrays.copyOf(rawbuf, rawbuf.length);
        SaveFile tsave = new SaveFile(rawbuf);

        // DEBUG BEGIN
        //skyjed.buffer.BooleanBuffer buf = tsave.slot1.buf;
        //skyjed.buffer.BooleanBuffer qbuf = tsave.qslot.buf;
        /*
        buf.seek(0x8A3F*8+2);
        for(int i=0;i<600;i++) if(buf.get()) System.out.println(skyjed.resources.SkyPokemonRes.getInstance().getValue(i));
        buf.seek(0x8A8A*8+2); // no-op
        for(int i=0;i<553;i++) if(buf.get()) System.out.println(skyjed.resources.SkyPokemonRes.getInstance().getValue(i));
        //*/
        /*
        System.out.println("searching");
        for(int i=0x8d30*8; i<0x8d30*8+200; i++) {
            buf.seek(i);
            System.out.print(String.format("%04x %d: ", i/8, i%8));
            System.out.println(skyjed.resources.SkyItemRes.getInstance().getValue(new HeldItemStorage(buf.view(HeldItemStorage.LENGTH)).items[0].id));
        }
        //*/
        //for(int p=0;p<4;p++) tsave.slot1.activePkmn.pkmns[p].dump();
        /*
        buf.seek(0x9dd2*8+2-RescueRequest.LENGTH);
        for(int p=0;p<31;p++) {
            RescueRequest req = new RescueRequest(buf.view(RescueRequest.LENGTH));
            req.dump();
        }
        //*/
        /*
        // changed when quicksaving in rescue
        buf.seek(0x9bc3*8+7);
        System.out.println("???: " + buf.getInt(20));
        //*/
        //for(int p=0;p<20;p++) tsave.qslot.pkmnQStorage.pkmns[p].dump();
        //tsave.slot1.adventureLog.dump();
        // DEBUG END

        return Arrays.equals(tbuf, tsave.toByteA());
    }

}
