package skyjed.buffer;

import skyjed.util.BitConverterLE;

public abstract class BooleanBuffer {

    protected int __position;
    protected int __size;

    protected BooleanBuffer(int size) {
        __position = 0;
        __size = size;
    }

    public int position() {
        return __position;
    }

    public int size() {
        return __size;
    }

    public int remaining() {
        return __size - __position;
    }

    public BooleanBuffer reset() {
        __position = 0;
        return this;
    }

    public BooleanBuffer seek(int new_position) {
        if(new_position > __size) throw new BufferOutOfBoundsException();
        __position = new_position;
        return this;
    }

    public BooleanBuffer skip(int length) {
        if(remaining() < length) throw new BufferOutOfBoundsException();
        __position += length;
        return this;
    }

    /// returns view of this buffer from current position and advances position
    public BooleanBuffer view(int length) {
        if(remaining() < length) throw new BufferOutOfBoundsException();
        BooleanBuffer ret = new BooleanBufferView(this, __position, length);
        __position += length;
        return ret;
    }

    public boolean get() {
        if(remaining() < 1) throw new BufferOutOfBoundsException();
        return aget(__position++);
    }

    public boolean[] get(int length) {
        if(remaining() < length) throw new BufferOutOfBoundsException();
        boolean[] ret = new boolean[length];
        for(int i=0; i<length; i++) {
            ret[i] = get();
        }
        return ret;
    }

    public byte getByte() {
        return BitConverterLE.packBits(get(8))[0];
    }

    public byte[] getBytes(int length) {
        return BitConverterLE.packBits(get(length*8));
    }

    public int getInt(int num_bits) {
        return BitConverterLE.packBitsInt(get(num_bits));
    }

    public int[] getInts(int length, int num_bits) {
        return BitConverterLE.packBitsInt(get(length*num_bits), num_bits);
    }

    public long getLong(int num_bits) {
        return BitConverterLE.packBitsLong(get(num_bits));
    }

    public long[] getLongs(int length, int num_bits) {
        return BitConverterLE.packBitsLong(get(length*num_bits), num_bits);
    }

    public void put(boolean b) {
        if(remaining() < 1) throw new BufferOutOfBoundsException();
        aput(__position++, b);
    }

    public void put(boolean[] src) {
        if(remaining() < src.length) throw new BufferOutOfBoundsException();
        for(int i=0; i<src.length; i++) {
            put(src[i]);
        }
    }

    public void putByte(byte val) {
        byte[] buf = new byte[1];
        buf[0] = val;
        put(BitConverterLE.splitBits(buf));
    }

    public void putBytes(byte[] vals) {
        put(BitConverterLE.splitBits(vals));
    }

    public void putInt(int val, int num_bits) {
        put(BitConverterLE.splitBitsInt(val, num_bits));
    }

    public void putInts(int[] vals, int num_bits) {
        put(BitConverterLE.splitBitsInt(vals, num_bits));
    }

    public void putLong(long val, int num_bits) {
        put(BitConverterLE.splitBitsLong(val, num_bits));
    }

    public void putLongs(long[] vals, int num_bits) {
        put(BitConverterLE.splitBitsLong(vals, num_bits));
    }

    protected abstract boolean aget(int index);
    protected abstract void aput(int index, boolean b);

}
