package skyjed.gui;

import net.java.dev.designgridlayout.DesignGridLayout;
import skyjed.gui.action.ManageEggAction;
import skyjed.gui.action.ManageItemsAction;
import skyjed.gui.action.ManagePokemonAction;
import skyjed.gui.action.ManagePokemonExAction;
import skyjed.gui.util.JPanelWindowHelper;
import skyjed.gui.util.JSkyTextField;
import skyjed.save.AdventureLog;
import skyjed.save.SaveSlot;
import skyjed.util.JNumberSpinner;

import javax.swing.*;
import javax.swing.text.JTextComponent;
import java.awt.*;

public class AdventureLogEditPanel extends JPanelWindowHelper {

    public static final int MAX_20 = (1 << 20) - 1;

    private AdventureLog advLog;
    private JNumberSpinner spnRankPoints;
    private JNumberSpinner spnPlayTime;
    private JNumberSpinner spnDungeonsCleared;
    private JNumberSpinner spnRecordNumOfOneFloorVictories;
    private JNumberSpinner spnFaintedInDungeons;
    private JNumberSpinner spnPokemonEggsHatched;
    private JNumberSpinner spnWonBigAtBigTreasure;
    private JNumberSpinner spnRecycled;
    private JNumberSpinner spnSkyGiftsSent;

    /**
     * Create the panel.
     */
    public AdventureLogEditPanel(AdventureLog log) {
        advLog = log;
        initialize();
    }

    /**
     * Initialize the contents of the frame.
     */
    private void initialize() {
        spnRankPoints = new JNumberSpinner(0, Integer.MAX_VALUE);
        spnRankPoints.setValue(advLog.rankPoints);
        spnPlayTime = new JNumberSpinner(0, Integer.MAX_VALUE);
        spnPlayTime.setValue(advLog.playTime);
        spnDungeonsCleared = new JNumberSpinner(0, MAX_20);
        spnDungeonsCleared.setValue(advLog.dungeonsCleared);
        spnRecordNumOfOneFloorVictories = new JNumberSpinner(0, MAX_20);
        spnRecordNumOfOneFloorVictories.setValue(advLog.recordNumOfOneFloorVictories);
        spnFaintedInDungeons = new JNumberSpinner(0, MAX_20);
        spnFaintedInDungeons.setValue(advLog.faintedInDungeons);
        spnPokemonEggsHatched = new JNumberSpinner(0, MAX_20);
        spnPokemonEggsHatched.setValue(advLog.pokemonEggsHatched);
        spnWonBigAtBigTreasure = new JNumberSpinner(0, MAX_20);
        spnWonBigAtBigTreasure.setValue(advLog.wonBigAtBigTreasure);
        spnRecycled = new JNumberSpinner(0, MAX_20);
        spnRecycled.setValue(advLog.recycled);
        spnSkyGiftsSent = new JNumberSpinner(0, MAX_20);
        spnSkyGiftsSent.setValue(advLog.skyGiftsSent);

        DesignGridLayout layout = new DesignGridLayout(this);
        layout.row().grid(label("Rank Points"))        .add(spnRankPoints)         .grid(label("Play Time (s)"))     .add(spnPlayTime);
        layout.row().grid(label("Dungeons Cleared"))   .add(spnDungeonsCleared)    .grid(label("#1-floor victories")).add(spnRecordNumOfOneFloorVictories);
        layout.row().grid(label("Fainted in Dungeons")).add(spnFaintedInDungeons)  .grid(label("Eggs Hatched"))      .add(spnPokemonEggsHatched);
        layout.row().grid(label("Won Big"))            .add(spnWonBigAtBigTreasure).grid(label("Recycled"))          .add(spnRecycled);
        layout.row().grid(label("Sky Gifts Sent"))     .add(spnSkyGiftsSent)       .grid()                           .empty();
    }

    public AdventureLog getSave() {
        advLog.rankPoints = spnRankPoints.getValueInt();
        advLog.playTime = spnPlayTime.getValueLong();
        advLog.dungeonsCleared = spnDungeonsCleared.getValueInt();
        advLog.recordNumOfOneFloorVictories = spnRecordNumOfOneFloorVictories.getValueInt();
        advLog.faintedInDungeons = spnFaintedInDungeons.getValueInt();
        advLog.pokemonEggsHatched = spnPokemonEggsHatched.getValueInt();
        advLog.wonBigAtBigTreasure = spnWonBigAtBigTreasure.getValueInt();
        advLog.recycled = spnRecycled.getValueInt();
        advLog.skyGiftsSent = spnSkyGiftsSent.getValueInt();
        return advLog;
    }

}
