package skyjed.gui;

import java.awt.Dimension;

import javax.swing.*;

import net.java.dev.designgridlayout.DesignGridLayout;
import skyjed.gui.action.ILoadStoreValues;
import skyjed.gui.action.UpdateItemAction;
import skyjed.gui.util.WindowHelper;
import skyjed.resources.PropertyItem;
import skyjed.resources.SkyPokemonRes;
import skyjed.save.SkyEgg;
import skyjed.util.ArrayListModel;
import skyjed.util.JNumberSpinner;

public class ManageEggWindow extends WindowHelper implements ILoadStoreValues<SkyEgg> {

    private static final int DAYS_TO_HATCH_MAX = 255;

    private JFrame frame;

    private JCheckBox chkBoxIsFemale;
    private JComboBox<PropertyItem> cmbSpecies;
    private JNumberSpinner spnDaysToHatch;

    private Action updateEggAction;

    public void loadValues(SkyEgg egg) {
        if(egg == null) {
            egg = new SkyEgg();
        }
        chkBoxIsFemale.setSelected(egg.pkmnno.isfemale);
        cmbSpecies.setSelectedIndex(egg.pkmnno.no);
        spnDaysToHatch.setValue(egg.daysToHatch);
    }

    public void storeValues(SkyEgg egg) {
        // Nothing selected - do nothing
        if(egg == null) return;
        egg.pkmnno.isfemale = chkBoxIsFemale.isSelected();
        egg.pkmnno.no = cmbSpecies.getSelectedIndex();
        egg.daysToHatch = spnDaysToHatch.getValueInt();
    }

    /**
     * Create the window.
     */
    public ManageEggWindow(SkyEgg egg) {
        updateEggAction = new UpdateItemAction<>("Apply", egg, this);
        initialize();
        frame.setVisible(true);
        loadValues(egg);
    }

    /**
     * Initialize the contents of the frame.
     */
    private void initialize() {
        frame = new JFrame();
        frame.setTitle("Manage Egg");
        frame.setLocationByPlatform(true);
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

        chkBoxIsFemale = new JCheckBox("Is Female");
        cmbSpecies = new JComboBox<PropertyItem>();
        cmbSpecies.setModel(new ArrayListModel<PropertyItem>(SkyPokemonRes.getInstance().getPropertyList()));

        spnDaysToHatch = new JNumberSpinner(DAYS_TO_HATCH_MAX);

        JButton btnUpdateEgg = new JButton(updateEggAction);
        frame.getRootPane().setDefaultButton(btnUpdateEgg);

        JPanel contentPane = new JPanel();

        DesignGridLayout layout = new DesignGridLayout(contentPane);
        layout.row().center().withOwnRowWidth().add(chkBoxIsFemale);
        layout.row().grid(label("Species"))      .add(cmbSpecies);
        layout.row().grid(label("Days to Hatch")).add(spnDaysToHatch);
        layout.emptyRow();
        layout.row().right().add(btnUpdateEgg);

        frame.setContentPane(contentPane);
        frame.pack();
        frame.setMinimumSize(new Dimension(frame.getWidth()+10, frame.getHeight()+10));
    }

}
