package skyjed.gui;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.java.dev.designgridlayout.DesignGridLayout;
import skyjed.gui.action.ILoadStoreValues;
import skyjed.gui.action.UpdateSelectedItemsInJListAction;
import skyjed.gui.util.WindowHelper;
import skyjed.resources.PropertyItem;
import skyjed.resources.SkyItemRes;
import skyjed.save.SkyItem;
import skyjed.save.util.LengthMismatchException;
import skyjed.save.util.SkyBufferUtils;
import skyjed.save.util.SkyClipboardInfo;
import skyjed.util.*;

public class ManageItemsWindow extends WindowHelper implements ILoadStoreValues<SkyItem> {

    private static final int ITEM_PARAM_MAX = 100; // TODO boxes have items in param
    private static final int ITEM_HELDBY_MAX = 4;

    private JFrame frame;

    private JList<SkyItem> itemList;
    private String note;
    private boolean enableFlags;
    private boolean enableParams;

    private JCheckBox chkBoxIsValid;
    private JComboBox<PropertyItem> cmbItemName;
    private JNumberSpinner spnParam;
    private JNumberSpinner spnHeldBy;
    private JBooleanLabel[] flagsList = new JBooleanLabel[SkyItem.LEN_FLAGS];

    private Action updateItemAction;

    private ListSelectionListener listSelectionListener = new ListSelectionListener(){
        public void valueChanged(ListSelectionEvent ev) {
            if(ev.getValueIsAdjusting()) return;
            loadValues(itemList.getSelectedValue());
        }
    };

    private MouseListener listMouseListener = new PopupMouseAdapter() {
        private SkyClipboardInfo clipInfo;

        private ActionListener popupMenuActionListener = new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                try {
                    List<SkyItem> items = itemList.getSelectedValuesList();
                    // Nothing selected - do nothing
                    if(items.size() == 0) return;
                    if(items.size() == 1) {
                        // Single selection
                        SkyItem item = items.get(0);
                        switch(ev.getActionCommand()) {
                            case "Popup->Clear":
                                SkyBufferUtils.clear(item);
                                break;
                            case "Popup->Cut":
                                SkyBufferUtils.cut(item);
                                break;
                            case "Popup->Copy":
                                SkyBufferUtils.copy(item);
                                break;
                            case "Popup->Paste":
                                if(clipInfo.type == SkyClipboardInfo.TYPE_SINGLE) {
                                    SkyBufferUtils.paste(item);
                                } else { // this is hacky
                                    int idxHead = itemList.getSelectedIndex();
                                    int idxTail = idxHead + clipInfo.arrayLength;
                                    if(idxTail > itemList.getModel().getSize()) {
                                        idxTail = itemList.getModel().getSize();
                                    }
                                    itemList.setSelectionInterval(idxHead, idxTail-1);
                                    try {
                                        SkyBufferUtils.pasteList(itemList.getSelectedValuesList());
                                    } finally {
                                        itemList.setSelectedIndex(idxHead);
                                    }
                                }
                                break;
                        }
                    } else {
                        // Multiple selection
                        switch(ev.getActionCommand()) {
                            case "Popup->Clear":
                                SkyBufferUtils.clearList(items);
                                break;
                            case "Popup->Cut":
                                SkyBufferUtils.cutList(items);
                                break;
                            case "Popup->Copy":
                                SkyBufferUtils.copyList(items);
                                break;
                            case "Popup->Paste":
                                if(clipInfo.type == SkyClipboardInfo.TYPE_ARRAY) {
                                    SkyBufferUtils.pasteList(items);
                                } else {
                                    for(SkyItem item : items) {
                                        SkyBufferUtils.paste(item);
                                    }
                                }
                                break;
                        }
                    }
                    // refresh the list
                    itemList.repaint();
                } catch (LengthMismatchException e) {
                    ExceptionHandler.handleException(e, frame);
                }
            }
        };

        protected void popUp(MouseEvent e) {
            int index = itemList.locationToIndex(e.getPoint());
            if(itemList.isSelectedIndex(index) == false) {
                itemList.setSelectedIndex(index);
            }

            JPopupMenu mnPopup = new JPopupMenu();

            JMenuItem mntmClear = new JMenuItem("Clear");
            mntmClear.setActionCommand("Popup->Clear");
            mntmClear.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmClear);

            JMenuItem mntmCut = new JMenuItem("Cut");
            mntmCut.setActionCommand("Popup->Cut");
            mntmCut.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmCut);

            JMenuItem mntmCopy = new JMenuItem("Copy");
            mntmCopy.setActionCommand("Popup->Copy");
            mntmCopy.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmCopy);

            JMenuItem mntmPaste = new JMenuItem("Paste");
            clipInfo = SkyBufferUtils.getInfoForMime(SkyItem.MIME_TYPE);
            mntmPaste.setEnabled(clipInfo.type != SkyClipboardInfo.TYPE_NONE);
            mntmPaste.setActionCommand("Popup->Paste");
            mntmPaste.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmPaste);

            mnPopup.show(e.getComponent(), e.getX(), e.getY());
        }
    };

    public void loadValues(SkyItem item) {
        if(item == null) {
            item = new SkyItem();
        }
        chkBoxIsValid.setSelected(item.isvalid);
        cmbItemName.setSelectedIndex(item.id);
        spnParam.setValue(item.param);
        spnHeldBy.setValue(item.heldby);
        for(int i=0; i<flagsList.length; i++) {
            flagsList[i].setBool(item.flags[i]);
        }
    }

    public void storeValues(SkyItem item) {
        // Nothing selected - do nothing
        if(item == null) return;
        item.isvalid = chkBoxIsValid.isSelected();
        item.id = cmbItemName.getSelectedIndex();
        item.param = spnParam.getValueInt();
        item.heldby = spnHeldBy.getValueInt();
        for(int i=0; i<flagsList.length; i++) {
            item.flags[i] = flagsList[i].getBool();
        }
    }

    /**
     * Create the window.
     */
    public ManageItemsWindow(SkyItem[] itemArray, String note, boolean enableFlags, boolean enableParams) {
        itemList = new JList<SkyItem>();
        itemList.setModel(new ArrayListModel<SkyItem>(itemArray));
        updateItemAction = new UpdateSelectedItemsInJListAction<>("Update Item", itemList, this);
        this.note = note;
        this.enableFlags = enableFlags;
        this.enableParams = enableParams;
        initialize();
        frame.setVisible(true);
    }

    /**
     * Initialize the contents of the frame.
     */
    private void initialize() {
        frame = new JFrame();
        frame.setTitle("Manage Items");
        frame.setLocationByPlatform(true);
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

        itemList.addListSelectionListener(listSelectionListener);
        itemList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        itemList.addMouseListener(listMouseListener);

        chkBoxIsValid = new JCheckBox("Is Valid");
        chkBoxIsValid.setEnabled(enableFlags);
        cmbItemName = new JComboBox<PropertyItem>();
        cmbItemName.setModel(new ArrayListModel<PropertyItem>(SkyItemRes.getInstance().getPropertyList()));

        spnParam = new JNumberSpinner(ITEM_PARAM_MAX);
        spnParam.setEnabled(enableParams);
        spnHeldBy = new JNumberSpinner(ITEM_HELDBY_MAX);
        spnHeldBy.setEnabled(enableFlags);

        JPanel flagsPanel = new JPanel();
        FlowLayout flowLayout = (FlowLayout) flagsPanel.getLayout();
        flowLayout.setAlignment(FlowLayout.LEFT);
        flowLayout.setVgap(0);
        flowLayout.setHgap(0);
        for(int i=0; i<flagsList.length; i++) {
            flagsList[i] = new JBooleanLabel();
            flagsList[i].setEnabled(enableFlags);
            flagsPanel.add(flagsList[i]);
        }
        flagsList[0].setToolTipText("is to buy");
        flagsList[2].setToolTipText("is sticky");
        flagsList[3].setToolTipText("is set throwable");

        JTextArea txtrInfo = null;
        if(note != null) {
            txtrInfo = new JTextArea();
            txtrInfo.setFont(UIManager.getFont("TextField.font"));
            txtrInfo.setBackground(frame.getBackground());
            txtrInfo.setText(note);
            txtrInfo.setEditable(false);
        }

        JButton btnUpdateItem = new JButton(updateItemAction);
        frame.getRootPane().setDefaultButton(btnUpdateItem);

        JPanel contentPane = new JPanel();
        JPanel mainContentPane = new JPanel();

        DesignGridLayout topLayout = new DesignGridLayout(contentPane);
        topLayout.disableSmartVerticalResize();
        topLayout.row().grid().add(new JScrollPane(itemList)).grid(2).add(mainContentPane);

        DesignGridLayout layout = new DesignGridLayout(mainContentPane);
        layout.margins(0);
        layout.row().center().withOwnRowWidth().add(chkBoxIsValid);
        layout.row().grid(label("Item"))   .add(cmbItemName);
        layout.row().grid(label("Param"))  .add(spnParam);
        layout.row().grid(label("Held By")).add(spnHeldBy);
        layout.row().grid(label("Flags"))  .add(flagsPanel);
        layout.emptyRow();
        if(txtrInfo != null) {
            layout.emptyRow();
            layout.row().left().add(txtrInfo);
        }
        layout.emptyRow();
        layout.row().right().add(btnUpdateItem);

        frame.setContentPane(contentPane);
        frame.pack();
        frame.setMinimumSize(new Dimension(frame.getWidth()+10, frame.getHeight()+10));
    }

}
