package skyjed.gui;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.JTextComponent;

import net.java.dev.designgridlayout.DesignGridLayout;
import skyjed.gui.action.ILoadStoreValues;
import skyjed.gui.action.UpdateSelectedItemsInJListAction;
import skyjed.gui.util.JSkyTextField;
import skyjed.gui.util.WindowHelper;
import skyjed.resources.*;
import skyjed.save.SkyPkmnEx;
import skyjed.save.util.LengthMismatchException;
import skyjed.save.util.SkyBufferUtils;
import skyjed.save.util.SkyClipboardInfo;
import skyjed.util.*;

public class ManagePokemonExWindow extends WindowHelper implements ILoadStoreValues<SkyPkmnEx> {

    private static final int PKMN_NAME_LENGTH = 10;
    private static final int PKMN_METFL_MAX = 99;
    private static final int PKMN_LVL_MAX = 100;
    private static final int PKMN_EXP_MAX = 9_999_999;
    private static final int PKMN_HP_MAX = 999;
    private static final int PKMN_IQ_MAX = 999;
    private static final int PKMN_ROSTER_NO_MAX = 720;
    private static final int PKMN_STAT_MAX = 255;

    private static final int PKMN_NUM_ATTACKS = 4;

    private JFrame frame;

    private JList<SkyPkmnEx> pkmnList;

    private JCheckBox chkBoxIsFemale;
    private JTextComponent txtName;
    private JComboBox<PropertyItem> cmbSpecies;
    private JComboBox<PropertyItem> cmbMetAt;
    private JNumberSpinner spnMetFloor;
    private JNumberSpinner spnLevel;
    private JNumberSpinner spnExp;
    private JNumberSpinner spnHP1;
    private JNumberSpinner spnHP2;
    private JNumberSpinner spnIQ;
    private JNumberSpinner spnRosterNo;
    private JNumberSpinner spnAtk;
    private JNumberSpinner spnSpAtk;
    private JNumberSpinner spnDef;
    private JNumberSpinner spnSpDef;
    private JComboBox<PropertyItem> cmbTactic;

    private AttackExEditPanel[] attackEditPanels;

    private JBooleanLabel[] iqMapList = new JBooleanLabel[SkyPkmnEx.LEN_IQ_MAP];

    private Action updatePokemonAction;

    private ListSelectionListener listSelectionListener = new ListSelectionListener(){
        public void valueChanged(ListSelectionEvent ev) {
            if(ev.getValueIsAdjusting()) return;
            loadValues(pkmnList.getSelectedValue());
        }
    };

    private MouseListener listMouseListener = new PopupMouseAdapter() {
        private SkyClipboardInfo clipInfo;

        private ActionListener popupMenuActionListener = new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                try {
                    List<SkyPkmnEx> pkmns = pkmnList.getSelectedValuesList();
                    // Nothing selected - do nothing
                    if(pkmns.size() == 0) return;
                    if(pkmns.size() == 1) {
                        // Single selection
                        SkyPkmnEx pkmn = pkmns.get(0);
                        switch(ev.getActionCommand()) {
                            case "Popup->Clear":
                                SkyBufferUtils.clear(pkmn);
                                break;
                            case "Popup->Cut":
                                SkyBufferUtils.cut(pkmn);
                                break;
                            case "Popup->Copy":
                                SkyBufferUtils.copy(pkmn);
                                break;
                            case "Popup->Paste":
                                if(clipInfo.type == SkyClipboardInfo.TYPE_SINGLE) {
                                    SkyBufferUtils.paste(pkmn);
                                } else { // this is hacky
                                    int idxHead = pkmnList.getSelectedIndex();
                                    int idxTail = idxHead + clipInfo.arrayLength;
                                    if(idxTail > pkmnList.getModel().getSize()) {
                                        idxTail = pkmnList.getModel().getSize();
                                    }
                                    pkmnList.setSelectionInterval(idxHead, idxTail-1);
                                    try {
                                        SkyBufferUtils.pasteList(pkmnList.getSelectedValuesList());
                                    } finally {
                                        pkmnList.setSelectedIndex(idxHead);
                                    }
                                }
                                break;
                        }
                    } else {
                        // Multiple selection
                        switch(ev.getActionCommand()) {
                            case "Popup->Clear":
                                SkyBufferUtils.clearList(pkmns);
                                break;
                            case "Popup->Cut":
                                SkyBufferUtils.cutList(pkmns);
                                break;
                            case "Popup->Copy":
                                SkyBufferUtils.copyList(pkmns);
                                break;
                            case "Popup->Paste":
                                if(clipInfo.type == SkyClipboardInfo.TYPE_ARRAY) {
                                    SkyBufferUtils.pasteList(pkmns);
                                } else {
                                    for(SkyPkmnEx pkmn : pkmns) {
                                        SkyBufferUtils.paste(pkmn);
                                    }
                                }
                                break;
                        }
                    }
                    // refresh the list
                    pkmnList.repaint();
                } catch (LengthMismatchException e) {
                    ExceptionHandler.handleException(e, frame);
                }
            }
        };

        protected void popUp(MouseEvent e) {
            int index = pkmnList.locationToIndex(e.getPoint());
            if(pkmnList.isSelectedIndex(index) == false) {
                pkmnList.setSelectedIndex(index);
            }

            JPopupMenu mnPopup = new JPopupMenu();

            JMenuItem mntmClear = new JMenuItem("Clear");
            mntmClear.setActionCommand("Popup->Clear");
            mntmClear.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmClear);

            JMenuItem mntmCut = new JMenuItem("Cut");
            mntmCut.setActionCommand("Popup->Cut");
            mntmCut.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmCut);

            JMenuItem mntmCopy = new JMenuItem("Copy");
            mntmCopy.setActionCommand("Popup->Copy");
            mntmCopy.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmCopy);

            JMenuItem mntmPaste = new JMenuItem("Paste");
            clipInfo = SkyBufferUtils.getInfoForMime(SkyPkmnEx.MIME_TYPE);
            mntmPaste.setEnabled(clipInfo.type != SkyClipboardInfo.TYPE_NONE);
            mntmPaste.setActionCommand("Popup->Paste");
            mntmPaste.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmPaste);

            mnPopup.show(e.getComponent(), e.getX(), e.getY());
        }
    };

    public void loadValues(SkyPkmnEx pkmn) {
        if(pkmn == null) {
            pkmn = new SkyPkmnEx();
        }
        chkBoxIsFemale.setSelected(pkmn.pkmnno.isfemale);
        txtName.setText(pkmn.name.toString());
        cmbSpecies.setSelectedIndex(pkmn.pkmnno.no);
        cmbMetAt.setSelectedIndex(pkmn.metat);
        spnMetFloor.setValue(pkmn.metfl);
        spnLevel.setValue(pkmn.lvl);
        spnExp.setValue(pkmn.exp);
        spnHP1.setValue(pkmn.hp1);
        spnHP2.setValue(pkmn.hp2);
        spnIQ.setValue(pkmn.iq);
        spnRosterNo.setValue(pkmn.roster_no);
        spnAtk.setValue(pkmn.stats[0]);
        spnSpAtk.setValue(pkmn.stats[1]);
        spnDef.setValue(pkmn.stats[2]);
        spnSpDef.setValue(pkmn.stats[3]);
        cmbTactic.setSelectedIndex(pkmn.tactic);
        for(int i=0; i<PKMN_NUM_ATTACKS; i++) {
            attackEditPanels[i].setAttackEx(pkmn.attacks[i]);
        }
        for(int i=0; i<iqMapList.length; i++) {
            iqMapList[i].setBool(pkmn.iqmap[i]);
        }
    }

    public void storeValues(SkyPkmnEx pkmn) {
        // Nothing selected - do nothing
        if(pkmn == null) return;
        pkmn.pkmnno.isfemale = chkBoxIsFemale.isSelected();
        pkmn.name.setString(txtName.getText());
        pkmn.pkmnno.no = cmbSpecies.getSelectedIndex();
        pkmn.metat = cmbMetAt.getSelectedIndex();
        pkmn.metfl = spnMetFloor.getValueInt();
        pkmn.lvl = spnLevel.getValueInt();
        pkmn.exp = spnExp.getValueInt();
        pkmn.hp1 = spnHP1.getValueInt();
        pkmn.hp2 = spnHP2.getValueInt();
        pkmn.iq = spnIQ.getValueInt();
        pkmn.roster_no = spnRosterNo.getValueInt();
        pkmn.stats[0] = spnAtk.getValueInt();
        pkmn.stats[1] = spnSpAtk.getValueInt();
        pkmn.stats[2] = spnDef.getValueInt();
        pkmn.stats[3] = spnSpDef.getValueInt();
        pkmn.tactic = cmbTactic.getSelectedIndex();
        for(int i=0; i<PKMN_NUM_ATTACKS; i++) {
            pkmn.attacks[i] = attackEditPanels[i].getAttackEx();
        }
        for(int i=0; i<iqMapList.length; i++) {
            pkmn.iqmap[i] = iqMapList[i].getBool();
        }
    }

    /**
     * Create the window.
     */
    public ManagePokemonExWindow(SkyPkmnEx[] pkmnArray) {
        pkmnList = new JList<SkyPkmnEx>();
        pkmnList.setModel(new ArrayListModel<SkyPkmnEx>(pkmnArray));
        updatePokemonAction = new UpdateSelectedItemsInJListAction<>("Update Pokemon", pkmnList, this);
        initialize();
        frame.setVisible(true);
    }

    /**
     * Initialize the contents of the frame.
     */
    private void initialize() {
        frame = new JFrame();
        frame.setTitle("Manage Active Pokemon");
        frame.setLocationByPlatform(true);
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

        pkmnList.addListSelectionListener(listSelectionListener);
        pkmnList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        pkmnList.addMouseListener(listMouseListener);

        chkBoxIsFemale = new JCheckBox("Is Female");

        cmbSpecies = new JComboBox<PropertyItem>();
        cmbSpecies.setModel(new ArrayListModel<PropertyItem>(SkyPokemonRes.getInstance().getPropertyList()));
        cmbMetAt = new JComboBox<PropertyItem>();
        cmbMetAt.setModel(new ArrayListModel<PropertyItem>(SkyLocationRes.getInstance().getPropertyList()));

        txtName = new JSkyTextField(PKMN_NAME_LENGTH);

        spnMetFloor = new JNumberSpinner(PKMN_METFL_MAX);
        spnLevel = new JNumberSpinner(PKMN_LVL_MAX);
        spnExp = new JNumberSpinner(PKMN_EXP_MAX);
        spnHP1 = new JNumberSpinner(PKMN_HP_MAX);
        spnIQ = new JNumberSpinner(PKMN_IQ_MAX);
        spnHP2 = new JNumberSpinner(PKMN_HP_MAX);
        spnRosterNo = new JNumberSpinner(PKMN_ROSTER_NO_MAX);
        spnAtk = new JNumberSpinner(PKMN_STAT_MAX);
        spnDef = new JNumberSpinner(PKMN_STAT_MAX);
        spnSpAtk = new JNumberSpinner(PKMN_STAT_MAX);
        spnSpDef = new JNumberSpinner(PKMN_STAT_MAX);

        JPanel iqMapPanelA = new JPanel();
        FlowLayout fl_iqMapPanelA = (FlowLayout) iqMapPanelA.getLayout();
        fl_iqMapPanelA.setAlignment(FlowLayout.LEFT);
        fl_iqMapPanelA.setVgap(0);
        fl_iqMapPanelA.setHgap(0);
        for(int i=0; i<42; i++) {
            iqMapList[i] = new JBooleanLabel();
            iqMapList[i].setToolTipText(SkyIQRes.getInstance().getValue(i));
            iqMapPanelA.add(iqMapList[i]);
        }

        JPanel iqMapPanelB = new JPanel();
        FlowLayout fl_iqMapPanelB = (FlowLayout) iqMapPanelB.getLayout();
        fl_iqMapPanelB.setAlignment(FlowLayout.LEFT);
        fl_iqMapPanelB.setVgap(0);
        fl_iqMapPanelB.setHgap(0);
        for(int i=42; i<iqMapList.length; i++) {
            iqMapList[i] = new JBooleanLabel();
            iqMapList[i].setToolTipText(SkyIQRes.getInstance().getValue(i));
            iqMapPanelB.add(iqMapList[i]);
        }

        cmbTactic = new JComboBox<PropertyItem>();
        cmbTactic.setModel(new ArrayListModel<PropertyItem>(SkyTacticRes.getInstance().getPropertyList()));

        attackEditPanels = new AttackExEditPanel[PKMN_NUM_ATTACKS];
        for(int i=0; i<PKMN_NUM_ATTACKS; i++) {
            attackEditPanels[i] = new AttackExEditPanel();
        }

        JButton btnUpdatePkmn = new JButton(updatePokemonAction);
        frame.getRootPane().setDefaultButton(btnUpdatePkmn);

        JPanel contentPane = new JPanel();
        JPanel mainContentPane = new JPanel();

        DesignGridLayout topLayout = new DesignGridLayout(contentPane);
        topLayout.disableSmartVerticalResize();
        topLayout.row().grid().add(new JScrollPane(pkmnList)).grid(3).add(mainContentPane);

        DesignGridLayout layout = new DesignGridLayout(mainContentPane);
        layout.margins(0);
        layout.row().center().withOwnRowWidth().add(chkBoxIsFemale);
        layout.row().grid(label("Species"))   .add(cmbSpecies);
        layout.row().grid(label("Met At"))    .add(cmbMetAt);
        layout.row().grid(label("Name"))      .add(txtName) .grid(label("Met At Floor")).add(spnMetFloor);
        layout.row().grid(label("Level"))     .add(spnLevel).grid(label("Exp"))         .add(spnExp);
        layout.row().grid(label("Current HP")).add(spnHP1)  .grid(label("IQ"))          .add(spnIQ);
        layout.row().grid(label("Base HP"))   .add(spnHP2)  .grid(label("Roster No"))   .add(spnRosterNo);
        layout.row().grid(label("Attack"))    .add(spnAtk)  .grid(label("Defense"))     .add(spnDef);
        layout.row().grid(label("Sp. Atk"))   .add(spnSpAtk).grid(label("Sp. Def"))     .add(spnSpDef);
        layout.row().grid(label("IQ Map"))    .add(iqMapPanelA);
        layout.row().grid()                   .add(iqMapPanelB);
        layout.row().grid(label("Tactic"))    .add(cmbTactic);
        for(int i=0; i<PKMN_NUM_ATTACKS; i++) {
            layout.row().grid(label(String.format("Attack %d", i+1))).add(attackEditPanels[i]);
        }
        layout.emptyRow();
        layout.row().right().add(btnUpdatePkmn);

        frame.setContentPane(contentPane);
        frame.pack();
        frame.setMinimumSize(new Dimension(frame.getWidth()+10, frame.getHeight()+10));
    }

}
