package skyjed.gui;

import java.awt.Dimension;
import java.awt.event.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.java.dev.designgridlayout.DesignGridLayout;
import skyjed.gui.action.ILoadStoreValues;
import skyjed.gui.action.UpdateSelectedItemsInJListAction;
import skyjed.gui.util.WindowHelper;
import skyjed.resources.PropertyItem;
import skyjed.resources.SkyPokemonRes;
import skyjed.save.SkyPkmnQ;
import skyjed.save.util.LengthMismatchException;
import skyjed.save.util.SkyBufferUtils;
import skyjed.save.util.SkyClipboardInfo;
import skyjed.util.*;

public class ManagePokemonQWindow extends WindowHelper implements ILoadStoreValues<SkyPkmnQ> {

    private static final int PKMN_LVL_MAX = 100;
    private static final int PKMN_EXP_MAX = 9_999_999;
    private static final int PKMN_HP_MAX = 999;
    private static final int PKMN_STAT_MAX = 255;

    private static final int PKMN_NUM_ATTACKS = 4;

    private JFrame frame;

    private JList<SkyPkmnQ> pkmnList;

    private JCheckBox chkBoxIsFemale;
    private JComboBox<PropertyItem> cmbSpecies;
    private JNumberSpinner spnLevel;
    private JNumberSpinner spnExp;
    private JNumberSpinner spnHP1;
    private JNumberSpinner spnHP2;
    private JNumberSpinner spnAtk;
    private JNumberSpinner spnSpAtk;
    private JNumberSpinner spnDef;
    private JNumberSpinner spnSpDef;

    private AttackQEditPanel[] attackEditPanels;

    private Action updatePokemonAction;

    private ListSelectionListener listSelectionListener = new ListSelectionListener(){
        public void valueChanged(ListSelectionEvent ev) {
            if(ev.getValueIsAdjusting()) return;
            loadValues(pkmnList.getSelectedValue());
        }
    };

    private MouseListener listMouseListener = new PopupMouseAdapter() {
        private SkyClipboardInfo clipInfo;

        private ActionListener popupMenuActionListener = new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                try {
                    List<SkyPkmnQ> pkmns = pkmnList.getSelectedValuesList();
                    // Nothing selected - do nothing
                    if(pkmns.size() == 0) return;
                    if(pkmns.size() == 1) {
                        // Single selection
                        SkyPkmnQ pkmn = pkmns.get(0);
                        switch(ev.getActionCommand()) {
                            case "Popup->Clear":
                                SkyBufferUtils.clear(pkmn);
                                break;
                            case "Popup->Cut":
                                SkyBufferUtils.cut(pkmn);
                                break;
                            case "Popup->Copy":
                                SkyBufferUtils.copy(pkmn);
                                break;
                            case "Popup->Paste":
                                if(clipInfo.type == SkyClipboardInfo.TYPE_SINGLE) {
                                    SkyBufferUtils.paste(pkmn);
                                } else { // this is hacky
                                    int idxHead = pkmnList.getSelectedIndex();
                                    int idxTail = idxHead + clipInfo.arrayLength;
                                    if(idxTail > pkmnList.getModel().getSize()) {
                                        idxTail = pkmnList.getModel().getSize();
                                    }
                                    pkmnList.setSelectionInterval(idxHead, idxTail-1);
                                    try {
                                        SkyBufferUtils.pasteList(pkmnList.getSelectedValuesList());
                                    } finally {
                                        pkmnList.setSelectedIndex(idxHead);
                                    }
                                }
                                break;
                        }
                    } else {
                        // Multiple selection
                        switch(ev.getActionCommand()) {
                            case "Popup->Clear":
                                SkyBufferUtils.clearList(pkmns);
                                break;
                            case "Popup->Cut":
                                SkyBufferUtils.cutList(pkmns);
                                break;
                            case "Popup->Copy":
                                SkyBufferUtils.copyList(pkmns);
                                break;
                            case "Popup->Paste":
                                if(clipInfo.type == SkyClipboardInfo.TYPE_ARRAY) {
                                    SkyBufferUtils.pasteList(pkmns);
                                } else {
                                    for(SkyPkmnQ pkmn : pkmns) {
                                        SkyBufferUtils.paste(pkmn);
                                    }
                                }
                                break;
                        }
                    }
                    // refresh the list
                    pkmnList.repaint();
                } catch (LengthMismatchException e) {
                    ExceptionHandler.handleException(e, frame);
                }
            }
        };

        protected void popUp(MouseEvent e) {
            int index = pkmnList.locationToIndex(e.getPoint());
            if(pkmnList.isSelectedIndex(index) == false) {
                pkmnList.setSelectedIndex(index);
            }

            JPopupMenu mnPopup = new JPopupMenu();

            JMenuItem mntmClear = new JMenuItem("Clear");
            mntmClear.setActionCommand("Popup->Clear");
            mntmClear.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmClear);

            JMenuItem mntmCut = new JMenuItem("Cut");
            mntmCut.setActionCommand("Popup->Cut");
            mntmCut.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmCut);

            JMenuItem mntmCopy = new JMenuItem("Copy");
            mntmCopy.setActionCommand("Popup->Copy");
            mntmCopy.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmCopy);

            JMenuItem mntmPaste = new JMenuItem("Paste");
            clipInfo = SkyBufferUtils.getInfoForMime(SkyPkmnQ.MIME_TYPE);
            mntmPaste.setEnabled(clipInfo.type != SkyClipboardInfo.TYPE_NONE);
            mntmPaste.setActionCommand("Popup->Paste");
            mntmPaste.addActionListener(this.popupMenuActionListener);
            mnPopup.add(mntmPaste);

            mnPopup.show(e.getComponent(), e.getX(), e.getY());
        }
    };

    public void loadValues(SkyPkmnQ pkmn) {
        if(pkmn == null) {
            pkmn = new SkyPkmnQ();
        }
        chkBoxIsFemale.setSelected(pkmn.pkmnno1.isfemale);
        cmbSpecies.setSelectedIndex(pkmn.pkmnno1.no);
        spnLevel.setValue(pkmn.lvl);
        spnExp.setValue(pkmn.exp);
        spnHP1.setValue(pkmn.hp1);
        spnHP2.setValue(pkmn.hp2);
        spnAtk.setValue(pkmn.stats[0]);
        spnSpAtk.setValue(pkmn.stats[1]);
        spnDef.setValue(pkmn.stats[2]);
        spnSpDef.setValue(pkmn.stats[3]);
        for(int i=0; i<PKMN_NUM_ATTACKS; i++) {
            attackEditPanels[i].setAttackQ(pkmn.attacks[i]);
        }
    }

    public void storeValues(SkyPkmnQ pkmn) {
        // Nothing selected - do nothing
        if(pkmn == null) return;
        pkmn.pkmnno1.isfemale = chkBoxIsFemale.isSelected();
        pkmn.pkmnno1.no = cmbSpecies.getSelectedIndex();
        pkmn.lvl = spnLevel.getValueInt();
        pkmn.exp = spnExp.getValueInt();
        pkmn.hp1 = spnHP1.getValueInt();
        pkmn.hp2 = spnHP2.getValueInt();
        pkmn.stats[0] = spnAtk.getValueInt();
        pkmn.stats[1] = spnSpAtk.getValueInt();
        pkmn.stats[2] = spnDef.getValueInt();
        pkmn.stats[3] = spnSpDef.getValueInt();
        for(int i=0; i<PKMN_NUM_ATTACKS; i++) {
            pkmn.attacks[i] = attackEditPanels[i].getAttackQ();
        }
    }

    /**
     * Create the window.
     */
    public ManagePokemonQWindow(SkyPkmnQ[] pkmnArray) {
        pkmnList = new JList<SkyPkmnQ>();
        pkmnList.setModel(new ArrayListModel<SkyPkmnQ>(pkmnArray));
        updatePokemonAction = new UpdateSelectedItemsInJListAction<>("Update Pokemon", pkmnList, this);
        initialize();
        frame.setVisible(true);
    }

    /**
     * Initialize the contents of the frame.
     */
    private void initialize() {
        frame = new JFrame();
        frame.setTitle("Manage Quicksave Pokemon");
        frame.setLocationByPlatform(true);
        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

        pkmnList.addListSelectionListener(listSelectionListener);
        pkmnList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        pkmnList.addMouseListener(listMouseListener);

        chkBoxIsFemale = new JCheckBox("Is Female");

        cmbSpecies = new JComboBox<PropertyItem>();
        cmbSpecies.setModel(new ArrayListModel<PropertyItem>(SkyPokemonRes.getInstance().getPropertyList()));

        spnLevel = new JNumberSpinner(PKMN_LVL_MAX);
        spnExp = new JNumberSpinner(PKMN_EXP_MAX);
        spnHP1 = new JNumberSpinner(PKMN_HP_MAX);
        spnHP2 = new JNumberSpinner(PKMN_HP_MAX);
        spnAtk = new JNumberSpinner(PKMN_STAT_MAX);
        spnDef = new JNumberSpinner(PKMN_STAT_MAX);
        spnSpAtk = new JNumberSpinner(PKMN_STAT_MAX);
        spnSpDef = new JNumberSpinner(PKMN_STAT_MAX);

        attackEditPanels = new AttackQEditPanel[PKMN_NUM_ATTACKS];
        for(int i=0; i<PKMN_NUM_ATTACKS; i++) {
            attackEditPanels[i] = new AttackQEditPanel();
        }

        JButton btnUpdatePkmn = new JButton(updatePokemonAction);
        frame.getRootPane().setDefaultButton(btnUpdatePkmn);

        JPanel contentPane = new JPanel();
        JPanel mainContentPane = new JPanel();

        DesignGridLayout topLayout = new DesignGridLayout(contentPane);
        topLayout.disableSmartVerticalResize();
        topLayout.row().grid().add(new JScrollPane(pkmnList)).grid(3).add(mainContentPane);

        DesignGridLayout layout = new DesignGridLayout(mainContentPane);
        layout.margins(0);
        layout.row().center().withOwnRowWidth().add(chkBoxIsFemale);
        layout.row().grid(label("Species"))   .add(cmbSpecies);
        layout.row().grid(label("Level"))     .add(spnLevel).grid(label("Exp"))    .add(spnExp);
        layout.row().grid(label("Current HP")).add(spnHP1)  .grid(label("Base HP")).add(spnHP2);
        layout.row().grid(label("Attack"))    .add(spnAtk)  .grid(label("Defense")).add(spnDef);
        layout.row().grid(label("Sp. Atk"))   .add(spnSpAtk).grid(label("Sp. Def")).add(spnSpDef);
        for(int i=0; i<PKMN_NUM_ATTACKS; i++) {
            layout.row().grid(label(String.format("Attack %d", i+1))).add(attackEditPanels[i]);
        }
        layout.emptyRow();
        layout.row().right().add(btnUpdatePkmn);

        frame.setContentPane(contentPane);
        frame.pack();
        frame.setMinimumSize(new Dimension(frame.getWidth()+10, frame.getHeight()+10));
    }

}
