package skyjed.gui;

import java.awt.Dimension;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.text.JTextComponent;

import net.java.dev.designgridlayout.DesignGridLayout;
import skyjed.gui.action.*;
import skyjed.gui.util.JPanelWindowHelper;
import skyjed.gui.util.JSkyTextField;
import skyjed.save.SaveSlot;
import skyjed.util.JNumberSpinner;

public class SaveEditPanel extends JPanelWindowHelper {

    public static final int TEAMNAME_LENGTH = 10;
    public static final int MAX_HELD_MONEY = 99_999;
    public static final int MAX_HELD_MONEY_EP = 99_999;
    public static final int MAX_HELD_MONEY_FR = 99_999;
    public static final int MAX_STORED_MONEY = 9_999_999;

    protected static final String HELD_ITEMS_WARNING_NOTE =
            "Warning: you can't enter\n" +
            "dungeons with more items\n" +
            "than your current maximum.";

    private SaveSlot saveSlot;
    private JTextComponent txtTeamName;
    private JNumberSpinner spnHeldMoney;
    private JNumberSpinner spnHeldMoneyEP;
    private JNumberSpinner spnHeldMoneyFR;
    private JNumberSpinner spnStoredMoney;
    private JNumberSpinner spnAdventuresHad;

    private Action openEggAction;
    private Action openShop1Action;
    private Action openShop2Action;
    private Action openCroagunkShopAction;
    private Action openStoredItemsAction;
    private Action openHeldItemsAction;
    private Action openHeldItemsEPAction;
    private Action openHeldItemsFRAction;
    private Action openPokemonAction;
    private Action openActivePokemonAction;
    private Action openActivePokemonEPAction;
    private Action openActivePokemonFRAction;

    /**
     * Create the panel.
     */
    public SaveEditPanel(SaveSlot save) {
        saveSlot = save;
        openEggAction = new ManageEggAction("Egg Info", save.egg);
        openShop1Action = new ManageItemsAction("Shop 1", save.itemShop1.items, null, false, true);
        openShop2Action = new ManageItemsAction("Shop 2", save.itemShop2.items, null, false, true);
        openCroagunkShopAction = new ManageItemsAction("Croagunk", save.croagunkShop.items, null, false, false);
        openStoredItemsAction = new ManageItemsAction("Stored Items", save.itemStorage.items, null, false, true);
        openHeldItemsAction = new ManageItemsAction("Held Items", save.heldItemStorage.items, HELD_ITEMS_WARNING_NOTE, true, true);
        openHeldItemsEPAction = new ManageItemsAction("Held Items EP", save.heldItemEPStorage.items, HELD_ITEMS_WARNING_NOTE, true, true);
        openHeldItemsFRAction = new ManageItemsAction("Held Items FR", save.heldItemFRStorage.items, HELD_ITEMS_WARNING_NOTE, true, true);
        openPokemonAction = new ManagePokemonAction("Pokemon", save.pkmnStorage.pkmns);
        openActivePokemonAction = new ManagePokemonExAction("Active Pokemon", save.activePkmn.pkmns);
        openActivePokemonEPAction = new ManagePokemonExAction("Active Pokemon EP", save.activePkmnEP.pkmns);
        openActivePokemonFRAction = new ManagePokemonExAction("Active Pokemon FR", save.activePkmnFR.pkmns);
        initialize();
    }

    /**
     * Initialize the contents of the frame.
     */
    private void initialize() {
        txtTeamName = new JSkyTextField(TEAMNAME_LENGTH);
        txtTeamName.setText(saveSlot.teamName.getString());

        spnAdventuresHad = new JNumberSpinner(Integer.MIN_VALUE, Integer.MAX_VALUE);
        spnAdventuresHad.setMinimumSize(new Dimension(120, 20)); // why??
        spnAdventuresHad.setPreferredSize(new Dimension(120, 20)); // why??
        spnAdventuresHad.setValue(saveSlot.adventuresHad);
        spnHeldMoney = new JNumberSpinner(MAX_HELD_MONEY);
        spnHeldMoney.setValue(saveSlot.heldMoney);
        spnHeldMoneyEP = new JNumberSpinner(MAX_HELD_MONEY_EP);
        spnHeldMoneyEP.setValue(saveSlot.heldMoneyEP);
        spnHeldMoneyFR = new JNumberSpinner(MAX_HELD_MONEY_FR);
        spnHeldMoneyFR.setValue(saveSlot.heldMoneyFR);
        spnStoredMoney = new JNumberSpinner(MAX_STORED_MONEY);
        spnStoredMoney.setValue(saveSlot.storedMoney);

        JButton btnEgg = new JButton(openEggAction);
        JButton btnShop1 = new JButton(openShop1Action);
        JButton btnShop2 = new JButton(openShop2Action);
        JButton btnCroagunkShop = new JButton(openCroagunkShopAction);
        JButton btnStoredItems = new JButton(openStoredItemsAction);
        JButton btnHeldItems = new JButton(openHeldItemsAction);
        JButton btnHeldItemsEP = new JButton(openHeldItemsEPAction);
        JButton btnHeldItemsFR = new JButton(openHeldItemsFRAction);
        JButton btnPokemon = new JButton(openPokemonAction);
        JButton btnActivePokemon = new JButton(openActivePokemonAction);
        JButton btnActivePokemonEP = new JButton(openActivePokemonEPAction);
        JButton btnActivePokemonFR = new JButton(openActivePokemonFRAction);

        DesignGridLayout layout = new DesignGridLayout(this);
        layout.row().grid(label("Team Name"))    .add(txtTeamName)   .grid(label("Adventures Had")).add(spnAdventuresHad);
        layout.row().grid(label("Held Money"))   .add(spnHeldMoney)  .grid(label("Stored Money"))  .add(spnStoredMoney);
        layout.row().grid(label("Held Money EP")).add(spnHeldMoneyEP).grid()                       .empty();
        layout.row().grid(label("Held Money FR")).add(spnHeldMoneyFR).grid()                       .empty();
        layout.emptyRow();
        layout.emptyRow();
        layout.row().right()                                          .add(btnEgg);
        layout.row().right().add(btnShop1)      .add(btnShop2)        .add(btnCroagunkShop);
        layout.row().right().add(btnStoredItems).add(btnHeldItems)    .add(btnHeldItemsEP);
        layout.row().right().add(btnPokemon)    .add(btnActivePokemon).add(btnActivePokemonEP);
        layout.emptyRow();
        layout.row().right()                    .add(btnHeldItemsFR).add(btnActivePokemonFR);
    }

    public SaveSlot getSave() {
        saveSlot.teamName.setString(txtTeamName.getText());
        saveSlot.heldMoney = spnHeldMoney.getValueInt();
        saveSlot.heldMoneyEP = spnHeldMoneyEP.getValueInt();
        saveSlot.heldMoneyFR = spnHeldMoneyFR.getValueInt();
        saveSlot.storedMoney = spnStoredMoney.getValueInt();
        saveSlot.adventuresHad = spnAdventuresHad.getValueInt();
        return saveSlot;
    }

}
