package skyjed.gui.util;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;

public class ByteTable extends JTable {

    private static final Font monospacefont = new Font(Font.MONOSPACED, Font.PLAIN, 12);

    private Byte[][] data;
    private DefaultTableModel model;

    private TableModelListener tableModelListener = new TableModelListener() {
        public void tableChanged(TableModelEvent ev) {
            if(ev.getType() != TableModelEvent.UPDATE) {
                System.err.println("Warning: unknown change to table");
                return;
            }
            if(ev.getFirstRow() == TableModelEvent.HEADER_ROW) {
                System.err.println("Warning: metadata row changed");
                return;
            }
            if(ev.getColumn() == TableModelEvent.ALL_COLUMNS) {
                System.err.println("Warning: all columns changed");
                return;
            }
            int col = ev.getColumn();
            for(int row = ev.getFirstRow(); row <= ev.getLastRow(); row++) {
                Object val = model.getValueAt(row, col);
                if(val.getClass() == Byte.class) {
                    // our change, no action needed
                    continue;
                }
                // user generated change
                Byte bval = data[row][col];
                if(bval != null) { // if it was null then it should not be changed
                    try {
                        bval = (byte) Integer.parseInt(val.toString());
                    } catch(NumberFormatException e) {}
                    data[row][col] = bval;
                }
                model.setValueAt(bval, row, col); // this will fire another event that we ignore
            }
        }
    };

    public ByteTable(Byte[][] data) {
        this(data, new DefaultTableModel(data, new String[data[0].length]));
    }

    public ByteTable(Byte[][] data, String[] columnNames) {
        this(data, new DefaultTableModel(data, columnNames));
    }

    private ByteTable(Byte[][] data, DefaultTableModel model) {
        this.data = data;
        this.model = model;
        model.addTableModelListener(tableModelListener);
        setModel(model);
        setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        setRowSelectionAllowed(false);
        setFont(monospacefont);
        FontMetrics fm = this.getFontMetrics(this.getFont());
        int w = fm.stringWidth("-128");
        this.setPreferredScrollableViewportSize(new Dimension(w, this.getPreferredScrollableViewportSize().height));
        for(int i=0; i<this.getColumnCount(); i++) {
            this.getColumnModel().getColumn(i).setPreferredWidth(w);
        }
    }

}
