package skyjed.gui.util;

import java.awt.Color;
import java.awt.Font;

import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.text.*;

import skyjed.util.SkyStringTrash;
import skyjed.util.SkyStringTrash.ParsedByte;

public class JSkyTextField extends JTextPane {

    private static final Font monospacefont = new Font(Font.MONOSPACED, Font.PLAIN, 12);

    private int limit;

    public JSkyTextField(int limit) {
        super();
        this.limit = limit;
        setDocument(new SkyDocument());
        setFont(monospacefont);

        // text field to copy L&F from
        JTextField textField = new JTextField(limit+1);
        setBorder(textField.getBorder());
        setMargin(textField.getMargin());
        setPreferredSize(textField.getPreferredSize());
    }

    private class SkyDocument extends DefaultStyledDocument {
        @Override
        public void remove(int offset, int length) throws BadLocationException {
            super.remove(offset, length);
        }

        @Override
        public void replace(int offset, int length, String str, AttributeSet attr) throws BadLocationException {
            // safeguards
            if(str == null) str = "";

            if(getLength() - length + str.length() <= 5*limit) {
                super.replace(offset, length, str, attr);
            }
        }

        @Override
        public void insertString(int offset, String str, AttributeSet attr) throws BadLocationException {
            // safeguards
            if(str == null) str = "";

            if(getLength() + str.length() <= 5*limit) {
                super.insertString(offset, str, attr);
            }
        }

        @Override
        protected void insertUpdate(DefaultDocumentEvent chng, AttributeSet attr) {
            SwingUtilities.invokeLater(runReparse);
            super.insertUpdate(chng, attr);
        }

        @Override
        protected void postRemoveUpdate(DefaultDocumentEvent chng) {
            SwingUtilities.invokeLater(runReparse);
            super.postRemoveUpdate(chng);
        }

        public String getText() {
            try {
                return getText(0, getLength());
            } catch(BadLocationException e) {
                // shouldn't happen
                e.printStackTrace();
                return "";
            }
        }

        private Runnable runReparse = new Runnable() {
            @Override
            public void run() {
                reparse();
            }
        };

        private void reparse() {
            ParsedByte[] parsed = (new SkyStringTrash(getText())).getParsedBytes();
            for(int i=0; i<parsed.length; i++) {
                ParsedByte pb = parsed[i];
                SimpleAttributeSet attr = new SimpleAttributeSet();

                if((pb.flags & ParsedByte.F_INVALID) != 0) {
                    StyleConstants.setForeground(attr, Color.RED);
                } else if((pb.flags & ParsedByte.F_TRASH) != 0) {
                    StyleConstants.setForeground(attr, Color.GRAY);
                } else {
                    StyleConstants.setForeground(attr, Color.BLACK);
                }
                setCharacterAttributes(pb.start_idx, pb.end_idx - pb.start_idx, attr, true);
            }
        }
    }

}
