package skyjed.save;

import skyjed.buffer.BooleanBuffer;

public class AdventureLog {

    private static final int LEN_RANK_POINTS = 32;
    private static final int LEN_SKIP0 = 38;
    private static final int LEN_PLAY_TIME = 32;
    private static final int LEN_DUNGEONS_CLEARED = 20;
    private static final int LEN_SKIP1 = 53;
    private static final int LEN_FLAGS = 59;
    private static final int LEN_SKIP2 = 56;
    private static final int LEN_POKEMON_JOINED = 14;
    private static final int LEN_KINDS_OF_POKEMON_BATTLED = 10;
    private static final int LEN_SKIP3 = 4;
    private static final int LEN_MOVES_LEARNED = 9;
    private static final int LEN_RECORD_NUM_OF_ONE_FLOOR_VICTORIES = 20;
    private static final int LEN_FAINTED_IN_DUNGEONS = 20;
    private static final int LEN_POKEMON_EGGS_HATCHED = 20;
    private static final int LEN_WON_BIG_AT_BIG_TREASURE = 20;
    private static final int LEN_RECYCLED = 20;
    private static final int LEN_SKY_GIFTS_SENT = 20;

    public int rankPoints; // signed
    public long playTime; // 32bit unsigned int would be sufficient
    public int dungeonsCleared;
    public boolean[] flags; // 23 read-write & 36 read-only
    public int pokemonJoined; // read-only
    public int kindsOfPokemonBattled; // read-only
    public int movesLearned; // read-only
    public int recordNumOfOneFloorVictories;
    public int faintedInDungeons;
    public int pokemonEggsHatched;
    public int wonBigAtBigTreasure;
    public int recycled;
    public int skyGiftsSent;

    public AdventureLog() {
        flags = new boolean[LEN_FLAGS];
    }

    public AdventureLog(BooleanBuffer buf) {
        this();
        load(buf);
    }

    public void load(BooleanBuffer buf) {
        rankPoints = buf.getInt(LEN_RANK_POINTS);
        buf.skip(LEN_SKIP0);
        playTime = buf.getLong(LEN_PLAY_TIME);
        dungeonsCleared = buf.getInt(LEN_DUNGEONS_CLEARED);
        buf.skip(LEN_SKIP1);
        for(int i=0; i<59; i++) flags[i] = buf.get();
        buf.skip(LEN_SKIP2);
        pokemonJoined = buf.getInt(LEN_POKEMON_JOINED);
        kindsOfPokemonBattled = buf.getInt(LEN_KINDS_OF_POKEMON_BATTLED);
        buf.skip(LEN_SKIP3);
        movesLearned = buf.getInt(LEN_MOVES_LEARNED);
        recordNumOfOneFloorVictories = buf.getInt(LEN_RECORD_NUM_OF_ONE_FLOOR_VICTORIES);
        faintedInDungeons = buf.getInt(LEN_FAINTED_IN_DUNGEONS);
        pokemonEggsHatched = buf.getInt(LEN_POKEMON_EGGS_HATCHED);
        wonBigAtBigTreasure = buf.getInt(LEN_WON_BIG_AT_BIG_TREASURE);
        recycled = buf.getInt(LEN_RECYCLED);
        skyGiftsSent = buf.getInt(LEN_SKY_GIFTS_SENT);
    }

    public void store(BooleanBuffer buf) {
        buf.putInt(rankPoints, LEN_RANK_POINTS);
        buf.skip(LEN_SKIP0);
        buf.putLong(playTime, LEN_PLAY_TIME);
        buf.putInt(dungeonsCleared, LEN_DUNGEONS_CLEARED);
        buf.skip(LEN_SKIP1);
        for(int i=0; i<59; i++) buf.put(flags[i]);
        buf.skip(LEN_SKIP2);
        buf.putInt(pokemonJoined, LEN_POKEMON_JOINED);
        buf.putInt(kindsOfPokemonBattled, LEN_KINDS_OF_POKEMON_BATTLED);
        buf.skip(LEN_SKIP3);
        buf.putInt(movesLearned, LEN_MOVES_LEARNED);
        buf.putInt(recordNumOfOneFloorVictories, LEN_RECORD_NUM_OF_ONE_FLOOR_VICTORIES);
        buf.putInt(faintedInDungeons, LEN_FAINTED_IN_DUNGEONS);
        buf.putInt(pokemonEggsHatched, LEN_POKEMON_EGGS_HATCHED);
        buf.putInt(wonBigAtBigTreasure, LEN_WON_BIG_AT_BIG_TREASURE);
        buf.putInt(recycled, LEN_RECYCLED);
        buf.putInt(skyGiftsSent, LEN_SKY_GIFTS_SENT);
    }

    public void dump() {
        System.out.println(String.format("rankPoints: %d", rankPoints));
        System.out.println(String.format("playTime: %d", playTime));
        System.out.println(String.format("dungeonsCleared: %d", dungeonsCleared));
        System.out.print("flags: ");
        for(int i=0; i<59; i++) System.out.print(flags[i]?1:0);
        System.out.println();
        System.out.println(String.format("pokemonJoined: %d", pokemonJoined));
        System.out.println(String.format("kindsOfPokemonBattled: %d", kindsOfPokemonBattled));
        System.out.println(String.format("movesLearned: %d", movesLearned));
        System.out.println(String.format("recordNumOfOneFloorVictories: %d", recordNumOfOneFloorVictories));
        System.out.println(String.format("faintedInDungeons: %d", faintedInDungeons));
        System.out.println(String.format("pokemonEggsHatched: %d", pokemonEggsHatched));
        System.out.println(String.format("wonBigAtBigTreasure: %d", wonBigAtBigTreasure));
        System.out.println(String.format("recycled: %d", recycled));
        System.out.println(String.format("skyGiftsSent: %d", skyGiftsSent));
    }

}
