package skyjed.save;

import java.util.Arrays;

import skyjed.buffer.BooleanBuffer;
import skyjed.buffer.BooleanBufferArray;
import skyjed.util.ArrayUtils;
import skyjed.util.BitConverterLE;

public class QuickSaveSlot {

    private static final int MAGIC_OFFSET = 0x20;
    private static final int MAGIC_BYTELENGTH = 8;
    private static final byte[] MAGIC_DATA = ArrayUtils.toBytes(0x82, 0xDD, 0x82, 0xB3, 0x82, 0xAB, 0x97, 0x6C);

    private static final int POKEMON_Q_STORAGE_OFFSET = 0x3170*8;
    private static final int POKEMON_Q_STORAGE_LENGTH = 20*8*429;

    // private (public for debugging)
    public boolean[] split_data;
    public BooleanBuffer buf;

    public PkmnQStorage pkmnQStorage;

    public QuickSaveSlot(byte[] data) {
        if(calcChecksum(data) != BitConverterLE.readInt32(data, 0)) {
            System.err.println("WARNING: checksum mismatch");
        }
        split_data = BitConverterLE.splitBits(data);
        buf = new BooleanBufferArray(split_data);
        data = null; // do not use later

        if(! Arrays.equals(MAGIC_DATA, buf.seek(MAGIC_OFFSET).getBytes(MAGIC_BYTELENGTH))) {
            System.err.println("WARNING: invalid magic");
        }

        pkmnQStorage = new PkmnQStorage(buf.seek(POKEMON_Q_STORAGE_OFFSET).view(POKEMON_Q_STORAGE_LENGTH));
    }

    public byte[] toByteA() {
        buf.seek(MAGIC_OFFSET).putBytes(MAGIC_DATA);

        pkmnQStorage.store(buf.seek(POKEMON_Q_STORAGE_OFFSET).view(POKEMON_Q_STORAGE_LENGTH));

        byte[] data = BitConverterLE.packBits(split_data);
        BitConverterLE.writeInt32(calcChecksum(data), data, 0);
        return data;
    }

    private static int calcChecksum(byte[] data) {
        int chksum = 0;
        for(int i=4; i<data.length; i+=4) {
            chksum += BitConverterLE.readInt32(data, i);
        }
        return chksum;
    }

    public static boolean checkMagic(byte[] data) {
        return Arrays.equals(MAGIC_DATA, ArrayUtils.copyFrom(data, MAGIC_OFFSET/8, MAGIC_BYTELENGTH)); // offset is in bits we need bytes here
    }

}
