package skyjed.save;

import skyjed.buffer.BooleanBuffer;
import skyjed.resources.SkyLocationRes;
import skyjed.save.util.SkyCopyPasteable;
import skyjed.util.SkyCharConv;
import skyjed.util.SkyStringTrash;

public class RescueRequest implements SkyCopyPasteable {

    public static final int LENGTH = 1217;
    public static final String MIME_TYPE = "application/x-sky-rescue-request";

    public int getLength() {
        return LENGTH;
    }

    public String getMimeType() {
        return MIME_TYPE;
    }

    private static final int LEN_UNK1A = 4;
    private static final int LEN_LOCATION = 8;
    private static final int LEN_FLOOR = 7;
    private static final int LEN_UNK1B = 157;
    private static final int BLEN_TEAMNAME = 10;
    private static final int LEN_UNK1C = 950;
    private static final int LEN_ATTEMPTS_LEFT = 5;
    private static final int LEN_UNK2 = 6;

    public boolean[] unk1a;
    public int location;
    public int floor;
    public boolean[] unk1b;
    public SkyStringTrash teamName;
    public boolean[] unk1c;
    public int attemptsLeft;
    public boolean[] unk2;

    public RescueRequest() {
        unk1a = new boolean[LEN_UNK1A];
        unk1b = new boolean[LEN_UNK1B];
        teamName = new SkyStringTrash();
        unk1c = new boolean[LEN_UNK1C];
        unk2 = new boolean[LEN_UNK2];
    }

    public RescueRequest(BooleanBuffer buf) {
        this();
        load(buf);
    }

    public void load(BooleanBuffer buf) {
        unk1a = buf.get(LEN_UNK1A);
        location = buf.getInt(LEN_LOCATION);
        floor = buf.getInt(LEN_FLOOR);
        unk1b = buf.get(LEN_UNK1B);
        teamName = SkyCharConv.decode(buf.getBytes(BLEN_TEAMNAME));
        unk1c = buf.get(LEN_UNK1C);
        attemptsLeft = buf.getInt(LEN_ATTEMPTS_LEFT);
        unk2 = buf.get(LEN_UNK2);
    }

    public void store(BooleanBuffer buf) {
        buf.put(unk1a);
        buf.putInt(location, LEN_LOCATION);
        buf.putInt(floor, LEN_FLOOR);
        buf.put(unk1b);
        buf.putBytes(SkyCharConv.encode(teamName));
        buf.put(unk1c);
        buf.putInt(attemptsLeft, LEN_ATTEMPTS_LEFT);
        buf.put(unk2);
    }

    public String toString() {
        return String.format("huh %d", 1);
    }

    public void dump() {
        System.out.print(String.format("attempts: %02d | ", attemptsLeft));
        System.out.print(String.format("location: %-30s | ", SkyLocationRes.getInstance().getValue(location) + " (" + floor + "F)"));
        System.out.print(String.format("team name: %-10s | ", teamName.getString()));
        for (int i = 0; i < LEN_UNK1A; i++) {
            System.out.print(unk1a[i] ? '1' : '0');
        }
        System.out.print(" | ");
        for (int i = 0; i < LEN_UNK1B; i++) {
            System.out.print(unk1b[i] ? '1' : '0');
        }
        System.out.print(" | ");
        for (int i = 0; i < LEN_UNK1C; i++) {
            System.out.print(unk1c[i] ? '1' : '0');
        }
        System.out.print(" | ");
        for (int i = 0; i < LEN_UNK2; i++) {
            System.out.print(unk2[i] ? '1' : '0');
        }
        System.out.println();
    }

}
