package skyjed.save;

import skyjed.buffer.BooleanBuffer;
import skyjed.save.util.SkyCopyPasteable;
import skyjed.util.SkyCharConv;
import skyjed.util.SkyStringTrash;

public class SkyPkmn implements SkyCopyPasteable {

    public static final int LENGTH = 362;
    public static final String MIME_TYPE = "application/x-sky-pokemon";

    public int getLength() { return LENGTH; }
    public String getMimeType() { return MIME_TYPE; }

    private static final int NUM_STATS = 4;
    private static final int NUM_ATTACKS = 4;

    private static final int LEN_LVL = 7;
    private static final int LEN_PKMN_NO = 11;
    private static final int LEN_METAT = 8;
    private static final int LEN_METFL = 7;
    private static final int LEN_EVO_LVL = 7;
    private static final int LEN_IQ = 10;
    private static final int LEN_HP = 10;
    private static final int LEN_STAT = 8;
    private static final int LEN_EXP = 24;
    public  static final int LEN_IQ_MAP = 69;
    private static final int LEN_TACTIC = 4;
    private static final int LEN_ATTACK = 21;
    private static final int BLEN_NAME = 10;

    public boolean isvalid;
    public int lvl;
    public SkyPkmnNo pkmnno;
    public int metat;
    public int metfl;
    public boolean unk1;
    public int evo1lvl;
    public int evo2lvl;
    public int iq;
    public int hp;
    public int[] stats; // atk spa def spd
    public int exp;
    public boolean[] iqmap;
    public int tactic;
    public SkyAttack[] attacks;
    public SkyStringTrash name;

    public SkyPkmn() {
        pkmnno = new SkyPkmnNo();
        stats = new int[NUM_STATS];
        iqmap = new boolean[LEN_IQ_MAP];
        attacks = new SkyAttack[NUM_ATTACKS];
        for(int i=0; i<NUM_ATTACKS; i++) {
            attacks[i] = new SkyAttack();
        }
        name = new SkyStringTrash();
    }

    public SkyPkmn(BooleanBuffer buf) {
        this();
        load(buf);
    }

    public void load(BooleanBuffer buf) {
        isvalid = buf.get();
        lvl = buf.getInt(LEN_LVL);
        pkmnno.load(buf.view(LEN_PKMN_NO));
        metat = buf.getInt(LEN_METAT);
        metfl = buf.getInt(LEN_METFL);
        unk1 = buf.get();
        evo1lvl = buf.getInt(LEN_EVO_LVL);
        evo2lvl = buf.getInt(LEN_EVO_LVL);
        iq = buf.getInt(LEN_IQ);
        hp = buf.getInt(LEN_HP);
        stats = buf.getInts(NUM_STATS, LEN_STAT);
        exp = buf.getInt(LEN_EXP);
        iqmap = buf.get(LEN_IQ_MAP);
        tactic = buf.getInt(LEN_TACTIC);
        for(int i=0; i<NUM_ATTACKS; i++) {
            attacks[i].load(buf.view(LEN_ATTACK));
        }
        name = SkyCharConv.decode(buf.getBytes(BLEN_NAME));
    }

    public void store(BooleanBuffer buf) {
        buf.put(isvalid);
        buf.putInt(lvl, LEN_LVL);
        pkmnno.store(buf.view(LEN_PKMN_NO));
        buf.putInt(metat, LEN_METAT);
        buf.putInt(metfl, LEN_METFL);
        buf.put(unk1);
        buf.putInt(evo1lvl, LEN_EVO_LVL);
        buf.putInt(evo2lvl, LEN_EVO_LVL);
        buf.putInt(iq, LEN_IQ);
        buf.putInt(hp, LEN_HP);
        buf.putInts(stats, LEN_STAT);
        buf.putInt(exp, LEN_EXP);
        buf.put(iqmap);
        buf.putInt(tactic, LEN_TACTIC);
        for(int i=0; i<NUM_ATTACKS; i++) {
            attacks[i].store(buf.view(LEN_ATTACK));
        }
        buf.putBytes(SkyCharConv.encode(name));
    }

    public String toString() {
        if(!isvalid) {
            return "----------";
        }
        return String.format("%s lvl%3d", name, lvl);
    }

}
