package skyjed.save;

import skyjed.buffer.BooleanBuffer;
import skyjed.save.util.SkyCopyPasteable;
import skyjed.util.SkyCharConv;
import skyjed.util.SkyStringTrash;

public class SkyPkmnEx implements SkyCopyPasteable {

    public static final int LENGTH = 546;
    public static final String MIME_TYPE = "application/x-sky-pokemon-ex";

    public int getLength() { return LENGTH; }
    public String getMimeType() { return MIME_TYPE; }

    private static final int NUM_STATS = 4;
    private static final int NUM_ATTACKS = 4;

    private static final int LEN_UNKS1 = 5;
    private static final int LEN_LVL = 7;
    private static final int LEN_METAT = 8;
    private static final int LEN_METFL = 7;
    private static final int LEN_UNKS2 = 1;
    private static final int LEN_IQ = 10;
    private static final int LEN_ROSTER_NO = 10;
    private static final int LEN_UNKS3 = 22;
    private static final int LEN_PKMN_NO = 11;
    private static final int LEN_HP1 = 10;
    private static final int LEN_HP2 = 10;
    private static final int LEN_STAT = 8;
    private static final int LEN_EXP = 24;
    private static final int LEN_ATTACK_EX = 29;
    private static final int LEN_UNKS4 = 105;
    public  static final int LEN_IQ_MAP = 69;
    private static final int LEN_TACTIC = 4;
    private static final int LEN_UNKS5 = 15;
    private static final int BLEN_NAME = 10;

    public boolean[] unksp1;
    public int lvl;
    public int metat;
    public int metfl;
    public boolean[] unksp2;
    public int iq;
    public int roster_no;
    public boolean[] unksp3;
    public SkyPkmnNo pkmnno;
    public int hp1; // current hp (updated when entering floor)
    public int hp2; // base hp (without bonuses)
    public int[] stats; // atk spa def spd
    public int exp;
    public SkyAttackEx[] attacks;
    public boolean[] unksp4;
    public boolean[] iqmap;
    public int tactic;
    public boolean[] unksp5;
    public SkyStringTrash name;

    public SkyPkmnEx() {
        unksp1 = new boolean[LEN_UNKS1];
        unksp2 = new boolean[LEN_UNKS2];
        unksp3 = new boolean[LEN_UNKS3];
        pkmnno = new SkyPkmnNo();
        stats = new int[NUM_STATS];
        attacks = new SkyAttackEx[NUM_ATTACKS];
        for(int i=0; i<NUM_ATTACKS; i++) {
            attacks[i] = new SkyAttackEx();
        }
        unksp4 = new boolean[LEN_UNKS4];
        iqmap = new boolean[LEN_IQ_MAP];
        unksp5 = new boolean[LEN_UNKS5];
        name = new SkyStringTrash();
    }

    public SkyPkmnEx(BooleanBuffer buf) {
        this();
        load(buf);
    }

    public void load(BooleanBuffer buf) {
        unksp1 = buf.get(LEN_UNKS1);
        lvl = buf.getInt(LEN_LVL);
        metat = buf.getInt(LEN_METAT);
        metfl = buf.getInt(LEN_METFL);
        unksp2 = buf.get(LEN_UNKS2);
        iq = buf.getInt(LEN_IQ);
        roster_no = buf.getInt(LEN_ROSTER_NO);
        unksp3 = buf.get(LEN_UNKS3);
        pkmnno.load(buf.view(LEN_PKMN_NO));
        hp1 = buf.getInt(LEN_HP1);
        hp2 = buf.getInt(LEN_HP2);
        stats = buf.getInts(NUM_STATS, LEN_STAT);
        exp = buf.getInt(LEN_EXP);
        for(int i=0; i<NUM_ATTACKS; i++) {
            attacks[i].load(buf.view(LEN_ATTACK_EX));
        }
        unksp4 = buf.get(LEN_UNKS4);
        iqmap = buf.get(LEN_IQ_MAP);
        tactic = buf.getInt(LEN_TACTIC);
        unksp5 = buf.get(LEN_UNKS5);
        name = SkyCharConv.decode(buf.getBytes(BLEN_NAME));
    }

    public void store(BooleanBuffer buf) {
        buf.put(unksp1);
        buf.putInt(lvl, LEN_LVL);
        buf.putInt(metat, LEN_METAT);
        buf.putInt(metfl, LEN_METFL);
        buf.put(unksp2);
        buf.putInt(iq, LEN_IQ);
        buf.putInt(roster_no, LEN_ROSTER_NO);
        buf.put(unksp3);
        pkmnno.store(buf.view(LEN_PKMN_NO));
        buf.putInt(hp1, LEN_HP1);
        buf.putInt(hp2, LEN_HP2);
        buf.putInts(stats, LEN_STAT);
        buf.putInt(exp, LEN_EXP);
        for(int i=0; i<NUM_ATTACKS; i++) {
            attacks[i].store(buf.view(LEN_ATTACK_EX));
        }
        buf.put(unksp4);
        buf.put(iqmap);
        buf.putInt(tactic, LEN_TACTIC);
        buf.put(unksp5);
        buf.putBytes(SkyCharConv.encode(name));
    }

    public String toString() {
        return String.format("%s lvl%3d", name, lvl);
    }

    public void dump() {
        System.out.print(String.format("%s lvl%3d | hp: %d/%d | stats: %d %d %d %d | id: %d | ", name, lvl, hp1, hp2, stats[0], stats[1], stats[2], stats[3], pkmnno.no));
        System.out.println(String.format("unksp1: %d %d %d %d %d | exp: %d", unksp1[0]?1:0, unksp1[1]?1:0, unksp1[2]?1:0, unksp1[3]?1:0, unksp1[4]?1:0, exp));
    }

}
