package skyjed.save;

import skyjed.buffer.BooleanBuffer;
import skyjed.resources.SkyPokemonRes;
import skyjed.save.util.SkyCopyPasteable;

public class SkyPkmnQ implements SkyCopyPasteable {

    public static final int BYTE_LENGTH = 429;
    public static final String MIME_TYPE = "application/x-sky-pokemon-q";

    public int getByteLength() { return BYTE_LENGTH; }
    public int getLength() { return 8*BYTE_LENGTH; }
    public String getMimeType() { return MIME_TYPE; }

    private static final int NUM_STATS = 4;
    private static final int NUM_ATTACKS = 4;

    private static final int BLEN_ATTACK_Q = 6;

    public SkyPkmnNo pkmnno1; // TODO don't know what is the difference (ditto?)
    public SkyPkmnNo pkmnno2;
    public int lvl;
    public int hp1; // current hp
    public int hp2; // base hp (without bonuses)
    public int hp_boost; // from IQ / items
    public int[] stats; // atk spa def spd
    public int exp;
    public SkyAttackQ[] attacks;

    public SkyPkmnQ() {
        pkmnno1 = new SkyPkmnNo();
        pkmnno2 = new SkyPkmnNo();
        stats = new int[NUM_STATS];
        attacks = new SkyAttackQ[NUM_ATTACKS];
        for(int i=0; i<NUM_ATTACKS; i++) {
            attacks[i] = new SkyAttackQ();
        }
    }

    public SkyPkmnQ(BooleanBuffer buf) {
        this();
        load(buf);
    }

    public void load(BooleanBuffer buf) {
        buf.seek(10*8);
        pkmnno1.load(buf.view(16));
        pkmnno2.load(buf.view(16));
        buf.seek(18*8);
        lvl = buf.getInt(8);
        buf.seek(24*8);
        hp1 = buf.getInt(16);
        hp2 = buf.getInt(16);
        hp_boost = buf.getInt(16);
        buf.seek(32*8);
        stats = buf.getInts(NUM_STATS, 8);
        exp = buf.getInt(32);
        buf.seek(337*8);
        for(int i=0; i<NUM_ATTACKS; i++) {
            attacks[i].load(buf.view(8*BLEN_ATTACK_Q));
        }
    }

    public void store(BooleanBuffer buf) {
        buf.seek(10*8);
        pkmnno1.store(buf.view(16));
        pkmnno2.store(buf.view(16));
        buf.seek(18*8);
        buf.putInt(lvl, 8);
        buf.seek(24*8);
        buf.putInt(hp1, 16);
        buf.putInt(hp2, 16);
        buf.putInt(hp_boost, 16);
        buf.seek(32*8);
        buf.putInts(stats, 8);
        buf.putInt(exp, 32);
        buf.seek(337*8);
        for(int i=0; i<NUM_ATTACKS; i++) {
            attacks[i].store(buf.view(8*BLEN_ATTACK_Q));
        }
    }

    public String toString() {
        return String.format("%s lvl%3d", SkyPokemonRes.getInstance().getValue(pkmnno1.no), lvl);
    }

    public void dump() {
        System.out.print(String.format("%s lvl%3d | hp: %d/%d+%d | stats: %d %d %d %d | ", SkyPokemonRes.getInstance().getValue(pkmnno1.no), lvl, hp1, hp2, hp_boost, stats[0], stats[1], stats[2], stats[3]));
        System.out.println(String.format("id: %d/%d | exp: %d", pkmnno1.no, pkmnno2.no, exp));
    }

}
