package skyjed.util;

public class BitConverterLE {

    public static int readInt32(byte[] buf, int off) {
        int b0 = buf[off+0] & 0xFF;
        int b1 = buf[off+1] & 0xFF;
        int b2 = buf[off+2] & 0xFF;
        int b3 = buf[off+3] & 0xFF;
        return b0 << 0 | b1 << 8 | b2 << 16 | b3 << 24;
    }

    public static void writeInt32(int val, byte[] buf, int off) {
        buf[off+0] = (byte) (val >> 0);
        buf[off+1] = (byte) (val >> 8);
        buf[off+2] = (byte) (val >> 16);
        buf[off+3] = (byte) (val >> 24);
    }

    public static byte[] packBits(boolean[] src) {
        byte[] dest = new byte[src.length/8];
        for(int i=0; i<dest.length; i++) {
            dest[i] = 0;
            for(int j=0; j<8; j++) {
                dest[i] |= (src[i*8+j]?1:0) << j;
            }
        }
        return dest;
    }

    public static boolean[] splitBits(byte[] src) {
        boolean[] dest = new boolean[src.length*8];
        for(int i=0; i<src.length; i++) {
            for(int j=0; j<8; j++) {
                dest[i*8+j] = ((src[i] >> j) & 1) != 0;
            }
        }
        return dest;
    }

    public static int[] packBitsInt(boolean[] src, int n) {
        int[] dest = new int[src.length/n];
        for(int i=0; i<dest.length; i++) {
            dest[i] = 0;
            for(int j=0; j<n; j++) {
                dest[i] |= (src[i*n+j]?1:0) << j;
            }
        }
        return dest;
    }

    public static int packBitsInt(boolean[] src) {
        return packBitsInt(src, src.length)[0];
    }

    public static boolean[] splitBitsInt(int[] src, int n) {
        boolean[] dest = new boolean[src.length*n];
        for(int i=0; i<src.length; i++) {
            for(int j=0; j<n; j++) {
                dest[i*n+j] = ((src[i] >> j) & 1) != 0;
            }
        }
        return dest;
    }

    public static boolean[] splitBitsInt(int src, int n) {
        return splitBitsInt(new int[]{src}, n);
    }

    public static long[] packBitsLong(boolean[] src, int n) {
        long[] dest = new long[src.length/n];
        for(int i=0; i<dest.length; i++) {
            dest[i] = 0;
            for(int j=0; j<n; j++) {
                dest[i] |= (long) (src[i*n+j]?1:0) << j;
            }
        }
        return dest;
    }

    public static long packBitsLong(boolean[] src) {
        return packBitsLong(src, src.length)[0];
    }

    public static boolean[] splitBitsLong(long[] src, int n) {
        boolean[] dest = new boolean[src.length*n];
        for(int i=0; i<src.length; i++) {
            for(int j=0; j<n; j++) {
                dest[i*n+j] = ((src[i] >> j) & 1) != 0;
            }
        }
        return dest;
    }

    public static boolean[] splitBitsLong(long src, int n) {
        return splitBitsLong(new long[]{src}, n);
    }

}
