package skyjed.util;

import java.awt.Toolkit;
import java.awt.datatransfer.*;
import java.io.IOException;

public final class ClipboardHelper {

    public static DataFlavor[] getAvailableDataFlavors() {
        // the requestor param of getContents is not currently used
        Transferable content = getSystemClipboard().getContents(null);
        if(content != null) return content.getTransferDataFlavors();
        return null;
    }

    public static String getPreferredMimeType() {
        DataFlavor[] arr = getAvailableDataFlavors();
        if(arr != null && arr.length > 0) return arr[0].getMimeType();
        return null;
    }

    /**
     * Place an object on the clipboard with given mime-type.
     * Example mime: text/plain; class=java.lang.String; charset=Unicode
     * Example mime: text/plain; class="[B"; charset=UTF-8
     * Example mime: application/x-java-serialized-object; class=java.lang.String
     */
    public static void setClipboardContents(final Object obj, final String mime) {
        try {
            final DataFlavor dFlavor = new DataFlavor(mime);
            Transferable contents = new Transferable() {
                public boolean isDataFlavorSupported(DataFlavor flavor) {
                    return flavor.equals(dFlavor);
                }
                public DataFlavor[] getTransferDataFlavors() {
                    return new DataFlavor[] { dFlavor };
                }
                public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
                    return obj;
                }
            };
            getSystemClipboard().setContents(contents, new ClipboardOwner() {
                public void lostOwnership(Clipboard clipboard, Transferable contents) {}
            });
        } catch (ClassNotFoundException e) {
            // only if not valid class specified in mime
            e.printStackTrace();
        }
    }

    public static void setClipboardString(String src) {
        setClipboardContents(src, "text/plain; class=java.lang.String; charset=Unicode");
    }

    /**
     * Get the object of specified mime-type residing on the clipboard; if none found return null.
     * Example mime: text/plain; class=java.lang.String; charset=Unicode
     * Example mime: text/plain; class="[B"; charset=UTF-8
     */
    public static Object getClipboardContents(String mime) {
        try {
            DataFlavor dFlavor = new DataFlavor(mime);
            // the requestor param of getContents is not currently used
            Transferable contents = getSystemClipboard().getContents(null);
            if(contents != null && contents.isDataFlavorSupported(dFlavor)) {
                return contents.getTransferData(dFlavor);
            }
        } catch (UnsupportedFlavorException e) {
            // highly unlikely since we checked it is supported
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            // only if not valid class specified in mime
            e.printStackTrace();
        } catch (IOException e) {
            // we will return null in that case
            e.printStackTrace();
        }
        return null;
    }

    public static String getClipboardString() {
        return (String) getClipboardContents("text/plain; class=java.lang.String; charset=Unicode");
    }

    private static Clipboard getSystemClipboard() {
        return Toolkit.getDefaultToolkit().getSystemClipboard();
    }

}
