package skyjed.util;

import java.util.ArrayList;

import skyjed.resources.SkyCharsetRes;

public class SkyStringTrash {

    private String string;
    private ParsedByte[] parsed;

    public SkyStringTrash(String string) {
        this.string = string;
        parsed = null;
    }

    public SkyStringTrash(char[] chars) {
        this(new String(chars));
    }

    public SkyStringTrash() {
        this("");
    }

    @Override
    public String toString() {
        StringBuilder strb = new StringBuilder();
        for(ParsedByte pb : getParsedBytes()) {
            if((pb.flags & ParsedByte.F_TRASH) == 0) {
                strb.append(pb.str);
            }
        }
        return strb.toString();
    }

    public final String getString() {
        return string;
    }

    public final void setString(String string) {
        this.string = string;
        // reset cached parse result so that it will be reparsed
        parsed = null;
    }

    public ParsedByte[] getParsedBytes() {
        if(parsed == null) {
            ArrayList<ParsedByte> parseBuf = new ArrayList<>();
            SkyCharsetRes charsetRes = SkyCharsetRes.getInstance();

            // whether we already had nil character ('\0') and are now in trash bytes
            boolean hadNil = false;

            int si = 0;
            while(si<string.length()) {
                ParsedByte pb;
                if(string.charAt(si) == '[') {
                    // special sequence
                    int si_end = string.indexOf(']', si+1);
                    if(si_end == -1) {
                        si_end = string.length();
                    } else {
                        si_end++;
                    }
                    pb = parseSubStr(charsetRes, si, si_end);
                    pb.flags |= ParsedByte.F_SPECIAL;
                    si = si_end;
                } else {
                    pb = parseSubStr(charsetRes, si, si+1);
                    si++;
                }

                if(pb.byt == 0) {
                    hadNil = true;
                }
                if(hadNil) {
                    pb.flags |= ParsedByte.F_TRASH;
                }
                parseBuf.add(pb);
            }

            parsed = parseBuf.toArray(new ParsedByte[parseBuf.size()]);
        }
        return parsed;
    }

    private ParsedByte parseSubStr(SkyCharsetRes charsetRes, int start_idx, int end_idx) {
        int flags = 0;
        String substr = string.substring(start_idx, end_idx);
        if(!charsetRes.isValidStr(substr)) {
            flags |= ParsedByte.F_INVALID;
        }
        return new ParsedByte(substr, start_idx, end_idx, charsetRes.str2byte(substr), flags);
    }

    public static class ParsedByte {

        public static final int F_INVALID = 0b0001;
        public static final int F_SPECIAL = 0b0010;
        public static final int F_TRASH   = 0b0100;

        public String str;
        public int start_idx;
        public int end_idx;
        public byte byt;
        public int flags;

        private ParsedByte(String str, int start_idx, int end_idx, byte byt, int flags) {
            this.str = str;
            this.start_idx = start_idx;
            this.end_idx = end_idx;
            this.byt = byt;
            this.flags = flags;
        }

    }

}
